package com.yeejoin.amos.bank.config;

import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.TaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.typroject.tyboot.component.emq.EmqKeeper;
import org.typroject.tyboot.component.emq.EmqxListener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yeejoin.amos.bank.TopographyNode;
import com.yeejoin.amos.bank.common.enums.DeviceStatusEnum;
import com.yeejoin.amos.bank.dao.IDeviceRecord;
import com.yeejoin.amos.bank.dao.repository.ITopographyLineRepository;
import com.yeejoin.amos.bank.entity.DeviceRecord;
import com.yeejoin.amos.bank.remote.client.RiskModelRemoteClient;
import com.yeejoin.amos.spc.business.dao.mapper.EquipmentMapper;
import com.yeejoin.amos.spc.business.remote.RemoteWebSocketServer;
import com.yeejoin.amos.spc.core.util.StringUtil;

import liquibase.integration.commandline.Main;

@Component
public class SelfSupportListener extends EmqxListener implements InitializingBean {

    private final Logger logger = LogManager.getLogger(SelfSupportListener.class);

    private static ObjectMapper mapper = new ObjectMapper();

    @Autowired
    private EmqKeeper emqKeeper;

    private static TaskExecutor taskExecutor;

    @Autowired
    EquipmentMapper equipmentMapper;

    @Autowired
    RiskModelRemoteClient riskModelRemoteClient;
    
    @Autowired
    IDeviceRecord iDeviceRecord;
    
    @Autowired
    ITopographyLineRepository nodeDao;
    
    @Autowired
    private RemoteWebSocketServer webSocketServer;


    /**
     * 监听主题
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        emqKeeper.subscript(CommTopic.SUPPORT_PING.getTopic(), 2, this);
    }

    /**
     * 接收主题下的消息
     */
    @Override
    public void processMessage(String s, MqttMessage mqttMessage) throws Exception {

        blueprintMsgtransfer(mqttMessage);
    }


    /**
     * 消息处理
     *
     * @param mqttMessage
     */
    private void blueprintMsgtransfer(MqttMessage mqttMessage) {

        try {
            String jsonStr = new String(mqttMessage.getPayload());
            System.out.println("----supprtmqtt received--jsonStr:" + jsonStr);
//            logger.error("----supprtmqtt received--jsonStr",jsonStr);
            JSONObject parseObject = JSON.parseObject(jsonStr);
            //logger.info("----supprtmqtt received sucess",parseObject);
            System.out.println("----supprtmqtt received sucess :"+ parseObject);
            Long eqpId = parseObject.getLong("eqpId");
            JSONArray jsonArray = parseObject.getJSONArray("metricDatas");
            JSONArray eqpConfigMap = parseObject.getJSONArray("eqpConfigMap");
            //获取ip
            String ipAddress = getIpAddress(eqpConfigMap);
            String eqpName = parseObject.getString("eqpName");
            
            if (ObjectUtils.isEmpty(jsonArray)) {
                return;
            }
            
            jsonArray.forEach(e -> {
                JSONObject object = (JSONObject) e;
                String string = object.getString("metricJson");
                if (!StringUtil.isNotEmpty(string)) {
                    return;
                }
                logger.error("---------shudu--string",string);
                System.out.println("----supprtmqtt chuli recording : "+ string);
                //{\"maxTime\":\"1\",\"packetLossRate\":\"0\",\"avgTime\":\"0\",\"minTime\":\"0\",\"发包数\":\"\",\"icmpPackSize\":\"32\",\"status\":\"通\"}
                JSONObject metricJson = JSONObject.parseObject(string);
               
				for (Map.Entry entry : metricJson.entrySet()) {
					// 指标名
					System.out.println("----start chuli device state : "+ entry);
					String remark = entry.getKey().toString();
					String statusName = entry.getValue().toString();
					if (remark.equals("status")) {
						DeviceRecord record = new DeviceRecord();
			            record.setDeviceId(eqpId);
						int statusByName = DeviceStatusEnum.getStatusByName(statusName);
						record.setDeviceStatus(statusByName);
						record.setMetricData(string);
					    record.setName(eqpName + " " + ipAddress);
						iDeviceRecord.save(record);
						TopographyNode node = equipmentMapper.queryNodeBySourceId(String.valueOf(eqpId));
						if(null!=node) {
							node.setNumber(statusByName);
							node.setState(statusByName);
							equipmentMapper.updateNodeState(node);
							String nodeDetail = (statusByName== 0 ? "[{\"name\":\"通信状态\",\"value\":\"正常\"}]":"[{\"name\":\"通信状态\",\"value\":\"异常\"}]");
							equipmentMapper.updateNodeDetail(node.getId(), nodeDetail);
						    try {
								webSocketServer.sendMessage("refresh", "self");
							} catch (Exception e1) {
								 logger.error("webSocketServer send self! ");
							}
						}
					}
				}
            });

        } catch (Exception e) {
            logger.error("消息流转报错.");
            logger.error(e.getMessage(), e);
            e.printStackTrace();
        }
    }
    
    public String getIpAddress(JSONArray eqpConfigMap) {
    	//配置信息
        if(null != eqpConfigMap && eqpConfigMap.size() > 0) {
        	for (Object object : eqpConfigMap) {
        		JSONObject  obj = (JSONObject) object;
        		String str = obj.getString("displayName");
        		if(str.contains("IP")) {
        			String ip = obj.getString("valueStr");
        			return ip;
        		}
			}
        }
        return null;
    }
}
